import 'package:equatable/equatable.dart';
import 'package:plugin_codelab_new/models/data_collection.dart';
import 'package:plugin_codelab_new/models/report_sub_type_model.dart';
import 'package:plugin_codelab_new/models/report_type_model.dart';
import 'package:flutter/cupertino.dart';

abstract class FeedState extends Equatable {
  const FeedState();
}

class FeedInitialState extends FeedState {
  @override
  List<Object> get props => [];
}

class FeedLoadingState extends FeedState {
  @override
  List<Object> get props => [];
}

class FeedReadyState extends FeedState {
  final String error;

  FeedReadyState({
    this.error,
  });

  @override
  List<Object> get props => [error];
}

class FeedErrorState extends FeedState {
  final String error;

  FeedErrorState({@required this.error});

  @override
  List<Object> get props => [error];
}

abstract class ReportsState extends Equatable {
  const ReportsState();
}

class ReportsInitialState extends ReportsState {
  @override
  List<Object> get props => [];
}

class ReportsLoadingState extends ReportsState {
  @override
  List<Object> get props => [];
}

class ReportsReadyState extends ReportsState {
  final DataCollection<ReportTypeModel> reportsTypeList;
  final DataCollection<ReportSubTypeModel> reportsSubTypeList;
  final String error;

  ReportsReadyState({
    this.reportsTypeList,
    this.reportsSubTypeList,
    this.error,
  });

  @override
  List<Object> get props => [reportsTypeList, reportsSubTypeList, error];
}

class ReportsErrorState extends ReportsState {
  final String error;

  ReportsErrorState({@required this.error});

  @override
  List<Object> get props => [error];
}
