import 'dart:convert';
import 'package:plugin_codelab_new/bloc/feed_cubit.dart';
import 'package:plugin_codelab_new/models/post_comment_model.dart';
import 'package:plugin_codelab_new/models/user_model.dart';
import 'package:plugin_codelab_new/service/http_service.dart';
import 'package:plugin_codelab_new/widgets/post.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:pusher_websocket_flutter/pusher.dart';
import 'package:flutter_screenutil/screen_util.dart';

import '../widgets/messages_item.dart';

class ChatScreen extends StatefulWidget {
  final String senderName;
  final String about;
  final String senderImage;
  final FeedPost post;
  final UserModel currentUser;

  const ChatScreen({
    @required this.senderName,
    @required this.about,
    @required this.senderImage,
    @required this.post,
    @required this.currentUser,
  });

  @override
  _ChatScreenState createState() => _ChatScreenState();
}

class _ChatScreenState extends State<ChatScreen> {
  bool isChatBuild;

  List<PostCommentModel> commentsList;

  Future<UserModel> getUserById(id) async {
    UserModel user = await feedCubit.feedRepository.fetchUserData(id);
    return user;
  }

  ScrollController _scrollController;
  FeedCubit feedCubit;
  Channel _channelComment;
  TextEditingController _textEditingController;

  int commentsLength;

  Future<List<PostCommentModel>> getCommentsList() async {
    commentsList = await feedCubit.feedRepository
        .fetchPostComments(this.widget.post.postData.id);
    return commentsList;
  }

  loadData() async {
    await getCommentsList().then(
      (data) {
        setState(
          () {
            commentsList = data;
          },
        );
      },
    );
  }

  @override
  void initState() {
    commentsLength = 0;
    _textEditingController = TextEditingController();
    feedCubit = BlocProvider.of<FeedCubit>(context);
    super.initState();
    commentsList = [];
    loadData();
    _initPusher();
    _scrollController = ScrollController();
  }

  @override
  void dispose() {
    _scrollController.dispose();
    _textEditingController.dispose();
    super.dispose();
  }

  Future<void> _initPusher() async {
    _channelComment = await Pusher.subscribe('comment');

    _channelComment.bind('create_comment', (onEvent) {
      List<dynamic> newComment = json.decode(onEvent.data);
      List<PostCommentModel> comments =
          newComment.map((i) => PostCommentModel.fromJson(i)).toList();
      if (mounted) {
        this.setState(() {
          if (comments[0].post.id == this.widget.post.postData.id) {
            commentsList.add(comments[0]);
            commentsLength = commentsList.length;
          }
        });
      }
    });

    _channelComment.bind('remove_comment', (onEvent) {
      int id = int.parse(onEvent.data);
      List<PostCommentModel> comments =
          commentsList.where((comment) => comment.id != id);
      if (mounted) {
        this.setState(() {
          commentsList = comments;
        });
      }
    });
  }

  void _sendMessage() {
    if (_textEditingController.text.isEmpty) return;

    HttpService("jhgvf").uploadComment(
        userId: this.widget.currentUser.id,
        postId: this.widget.post.postData.id,
        comment: _textEditingController.text);
    setState(() {
      _textEditingController.text = "";
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: PreferredSize(
        preferredSize: Size.fromHeight(70.0),
        child: AppBar(
          backgroundColor: Color(0xFF083356),
        ),
      ),
      backgroundColor: Colors.white,
      body: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          Expanded(
            child: ListView(
              children: [
                FeedPost(
                    postData: this.widget.post.postData,
                    currentUser: this.widget.post.currentUser,
                    likeId: this.widget.post.likeId,
                    dislikeId: this.widget.post.dislikeId,
                    saveId: this.widget.post.saveId,
                    images: this.widget.post.images,
                    commentsLength: commentsLength),
                Expanded(
                  child: ListView.builder(
                    shrinkWrap: true,
                    physics: ClampingScrollPhysics(),
                    controller: _scrollController,
                    itemCount: commentsList.length,
                    itemBuilder: (ctx, index) => commentsList[index].post.id ==
                            this.widget.post.postData.id
                        ? MessagesItem(
                            postCommentModel: commentsList[index],
                            isUserMassage: commentsList[index].user.id ==
                                    this.widget.currentUser.id
                                ? true
                                : false,
                            commentId: commentsList[index].id,
                          )
                        : null,
                  ),
                ),
              ],
            ),
          ),
          // Visibility(
          //   visible: _isTyping,
          //   child: Padding(
          //     padding: const EdgeInsets.all(8.0),
          //     child: Row(
          //       children: <Widget>[
          //         Text(
          //           '$_userNameTyping is typing',
          //           style: Theme.of(context).textTheme.headline6.copyWith(
          //                 color: Colors.black54,
          //                 fontWeight: FontWeight.w400,
          //                 fontSize: 14,
          //               ),
          //         ),
          //       ],
          //     ),
          //   ),
          // ),
          Container(
            color: Colors.grey.withAlpha(50),
            child: Padding(
              padding: const EdgeInsets.symmetric(horizontal: 10, vertical: 5),
              child: Row(
                children: <Widget>[
                  ClipOval(
                    child: Image.network(
                      this.widget.currentUser.profileImg,
                      width: ScreenUtil().setWidth(36),
                      height: ScreenUtil().setHeight(26),
                      fit: BoxFit.cover,
                    ),
                  ),
                  Expanded(
                    child: Container(
                      padding: const EdgeInsets.symmetric(horizontal: 10),
                      decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(50),
                        color: Colors.white,
                      ),
                      child: TextField(
                        onSubmitted: (_) {
                          _sendMessage();
                        },
                        controller: _textEditingController,
                        decoration: InputDecoration(
                          hintText: 'Enter your message...',
                          border: InputBorder.none,
                        ),
                      ),
                    ),
                  ),
                  Container(
                    child: ClipOval(
                        child: Material(
                      color: Color(0xFF3DD598),
                      child: InkWell(
                        splashColor: Color(0xFFFCA64F),
                        child: IconButton(
                          onPressed: _sendMessage,
                          icon: Icon(FontAwesomeIcons.telegramPlane),
                          color: Colors.white,
                          iconSize: 35,
                        ),
                      ),
                    )),
                  ),
                ],
              ),
            ),
          ),
          // MessageForm(
          //   postId: this.widget.post.postData.id,
          //   currentUser: this.widget.currentUser,
          // ),
        ],
      ),
    );
  }
}
