import 'dart:async';
import 'package:plugin_codelab_new/models/user_model.dart';
import 'package:plugin_codelab_new/service/http_service.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/screen_util.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';

class MessageForm extends StatefulWidget {
  final int postId;

  final UserModel currentUser;
  const MessageForm({
    @required this.postId,
    @required this.currentUser,
  });

  @override
  _MessageFormState createState() => _MessageFormState();
}

class _MessageFormState extends State<MessageForm> {
  TextEditingController _textEditingController;

  void _sendMessage() {
    if (_textEditingController.text.isEmpty) return;

    HttpService("jhgvf").uploadComment(
        userId: this.widget.currentUser.id,
        postId: this.widget.postId,
        comment: _textEditingController.text);
    setState(() {
      _textEditingController.text = "";
    });
  }

  @override
  void initState() {
    super.initState();
    _textEditingController = TextEditingController();
  }

  @override
  void dispose() {
    _textEditingController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: Colors.grey.withAlpha(50),
      child: Padding(
        padding: const EdgeInsets.symmetric(horizontal: 10, vertical: 5),
        child: Row(
          children: <Widget>[
            ClipOval(
              child: Image.network(
                this.widget.currentUser.profileImg,
                width: ScreenUtil().setWidth(36),
                height: ScreenUtil().setHeight(26),
                fit: BoxFit.cover,
              ),
            ),
            Expanded(
              child: Container(
                padding: const EdgeInsets.symmetric(horizontal: 10),
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(50),
                  color: Colors.white,
                ),
                child: TextField(
                  onSubmitted: (_) {
                    _sendMessage();
                  },
                  controller: _textEditingController,
                  decoration: InputDecoration(
                    hintText: 'Enter your message...',
                    border: InputBorder.none,
                  ),
                ),
              ),
            ),
            Container(
              child: ClipOval(
                  child: Material(
                color: Color(0xFF3DD598),
                child: InkWell(
                  splashColor: Color(0xFFFCA64F),
                  child: IconButton(
                    onPressed: _sendMessage,
                    icon: Icon(FontAwesomeIcons.telegramPlane),
                    color: Colors.white,
                    iconSize: 35,
                  ),
                ),
              )),
            ),
          ],
        ),
      ),
    );
  }
}
