import 'package:bubble/bubble.dart';
import 'package:plugin_codelab_new/bloc/feed_cubit.dart';
import 'package:plugin_codelab_new/models/post_comment_model.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_screenutil/screen_util.dart';
import 'package:timeago/timeago.dart' as timeago;

class MessagesItem extends StatefulWidget {
  const MessagesItem(
      {Key key,
      @required this.postCommentModel,
      @required this.isUserMassage,
      @required this.commentId})
      : super(key: key);

  final PostCommentModel postCommentModel;
  final bool isUserMassage;
  final int commentId;

  @override
  _MessagesItemState createState() => _MessagesItemState();
}

class _MessagesItemState extends State<MessagesItem> {
  bool deleteComment = false;
  FeedCubit feedCubit;

  @override
  void initState() {
    super.initState();
    feedCubit = BlocProvider.of<FeedCubit>(context);
  }

  @override
  Widget build(BuildContext context) {
    final date2 = DateTime.now();
    final difference =
        date2.difference(this.widget.postCommentModel.createdAt).inMinutes;
    final fifteenAgo =
        new DateTime.now().subtract(new Duration(minutes: difference));
    BubbleStyle styleSomebody = BubbleStyle(
      nip: BubbleNip.leftTop,
      elevation: 1,
      margin: BubbleEdges.only(
        top: ScreenUtil().setWidth(8),
        right: ScreenUtil().setWidth(10),
        left: ScreenUtil().setWidth(50),
      ),
      alignment: Alignment.topLeft,
    );
    BubbleStyle styleImageSomebody = BubbleStyle(
      color: Colors.white.withOpacity(0),
      elevation: 0,
      margin: BubbleEdges.only(
        top: ScreenUtil().setWidth(8),
      ),
      alignment: Alignment.topLeft,
    );
    AlertDialog alert = AlertDialog(
      content: Text("Are you sure you want to delete this comment?"),
      actions: [
        FlatButton(
          child: Text("Cancel"),
          onPressed: () {
            this.setState(() {
              deleteComment = true;
              Navigator.of(context).pop();
            });
          },
        ),
        FlatButton(
          child: Text("Ok"),
          onPressed: () {
            print(
                "his.widget.commentIdhis.widget.commentIdhis.widget.commentIdhis.widget.commentId" +
                    this.widget.commentId.toString());
            feedCubit.feedRepository
                .removeComment(this.widget.commentId)
                .then((value) => {
                      this.setState(() {
                        deleteComment = true;
                        Navigator.of(context).pop();
                      })
                    });
          },
        ),
      ],
    );
    return GestureDetector(
        onLongPress: () {
          if (this.widget.isUserMassage) {
            this.setState(() {
              deleteComment = true;
            });
            showDialog(
              context: context,
              builder: (BuildContext context) {
                return alert;
              },
            );
          }
        },
        child: Column(
          children: [
            Stack(
              children: [
                Bubble(
                  style: styleImageSomebody,
                  child: ClipOval(
                    child: Image.network(
                      this.widget.postCommentModel.user.profileImg,
                      width: ScreenUtil().setWidth(36),
                      height: ScreenUtil().setHeight(26),
                      fit: BoxFit.cover,
                    ),
                  ),
                ),
                Bubble(
                  style: styleSomebody,
                  color: deleteComment ? Colors.grey : Color(0xffE8ECEF),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Row(children: [
                        Text(this
                                .widget
                                .postCommentModel
                                .user
                                .firstName
                                .toString() +
                            " " +
                            this
                                .widget
                                .postCommentModel
                                .user
                                .lastName
                                .toString()),
                        Text(
                          " -" + timeago.format(fifteenAgo, locale: 'en_short'),
                          style: TextStyle(color: Color(0xff98A9BC)),
                        )
                      ]),
                      Text(this.widget.postCommentModel.user.about,
                          style: TextStyle(color: Color(0xff98A9BC))),
                      Text(""),
                      Text(
                        this.widget.postCommentModel.comment,
                        style: TextStyle(fontSize: ScreenUtil().setSp(16)),
                      ),
                    ],
                  ),
                ),
              ],
            ),
          ],
        ));
  }
}
