import 'dart:io';
import 'package:plugin_codelab_new/bloc/feed_cubit.dart';
import 'package:plugin_codelab_new/models/user_model.dart';
import 'package:plugin_codelab_new/models/media_source.dart';
import 'package:plugin_codelab_new/pages/source_page.dart';
import 'package:plugin_codelab_new/widgets/video_widget.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_screenutil/screenutil_init.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:giphy_picker/giphy_picker.dart';
import 'package:hashtagable/hashtagable.dart';
import 'package:cloudinary_public/cloudinary_public.dart';

import 'package:video_player/video_player.dart';

import 'package:flutter/cupertino.dart';
//multi image picker
import 'dart:async';
import 'package:multi_image_picker/multi_image_picker.dart';

class AspectRatioVideo extends StatefulWidget {
  AspectRatioVideo(this.controller);

  final VideoPlayerController controller;

  @override
  AspectRatioVideoState createState() => AspectRatioVideoState();
}

class AspectRatioVideoState extends State<AspectRatioVideo> {
  VideoPlayerController get controller => widget.controller;
  bool initialized = false;
  FeedCubit feedCubit;

  void _onVideoControllerUpdate() {
    if (!mounted) {
      return;
    }
    if (initialized != controller.value.initialized) {
      initialized = controller.value.initialized;
      setState(() {});
    }
  }

  @override
  void initState() {
    feedCubit = BlocProvider.of<FeedCubit>(context);
    controller.addListener(_onVideoControllerUpdate);
    super.initState();
  }

  @override
  void dispose() {
    controller.removeListener(_onVideoControllerUpdate);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (initialized) {
      return Center(
        child: AspectRatio(
          aspectRatio: controller.value?.aspectRatio,
          child: VideoPlayer(controller),
        ),
      );
    } else {
      return Container();
    }
  }
}

class SharePostPage extends StatefulWidget {
  const SharePostPage({
    Key key,
    @required this.currentUser,
  }) : super(key: key);

  final UserModel currentUser;
  @override
  _SharePostPageState createState() => _SharePostPageState();
}

class _SharePostPageState extends State<SharePostPage> {
  FeedCubit feedCubit;
  String url = "https://api.cloudinary.com/v1_1/phonedo/image/upload";
  String pluginTitle;
  GiphyGif _gif;
  final descriptionController = TextEditingController();
  final titleController = TextEditingController();
  final _formKey = GlobalKey<FormState>();
  final cloudinary = CloudinaryPublic('phonedo', 'gfe14pef', cache: false);
  var fileUrl;
  //multi image picker--
  List<Asset> images = [];
  // List<Asset>();
  String jsonUrl = "";

  @override
  void initState() {
    feedCubit = BlocProvider.of<FeedCubit>(context);
    super.initState();
  }

  Widget _buildGridView() {
    if (images != null) {
      return GridView.count(
        shrinkWrap: true,
        crossAxisCount: 2,
        children: List.generate(images.length, (index) {
          Asset asset = images[index];

          return AssetThumb(
            asset: asset,
            width: 450,
            height: 440,
          );
        }),
      );
    } else {
      return Container();
    }
  }

  Future<void> loadAssets() async {
    List<Asset> resultList = [];
    // List<Asset>();
    try {
      resultList = await MultiImagePicker.pickImages(
        maxImages: 4,
        enableCamera: true,
        selectedAssets: images,
        cupertinoOptions: CupertinoOptions(takePhotoIcon: "chat"),
        materialOptions: MaterialOptions(
          actionBarColor: "#083356",
          actionBarTitle: "Example App",
          allViewTitle: "All Photos",
          useDetailsView: false,
          selectCircleStrokeColor: "#000000",
        ),
      );
    } on Exception catch (e) {
      print(e);
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    setState(() {
      images = resultList;
    });
  }

  Future<void> getimageditor() async {
    // final geteditimage =
    Navigator.push(context, MaterialPageRoute(builder: (context) {
      return Text(
          "  // return ImageEditorPro//   appBarColor: Colors.blue // );");
    })).then((geteditimage) {
      if (geteditimage != null) {
        setState(() {
          images = geteditimage;
        });
      }
    }).catchError((er) {
      print(er);
    });
  }
//--multi image picker

//video--
  File fileMedia;
  MediaSource source;

  Future capture(MediaSource source) async {
    setState(() {
      this.source = source;
      this.fileMedia = null;
    });

    final result = await Navigator.of(context).push(
      MaterialPageRoute(
        builder: (context) => SourcePage(),
        settings: RouteSettings(
          arguments: source,
        ),
      ),
    );

    if (result == null) {
      return;
    } else {
      setState(() {
        fileMedia = result;
      });
    }
  }

//--video
  Future<UserModel> getCurrentUserProfile() async {
    int userId = 5;
    UserModel user = await feedCubit.feedRepository.fetchUserData(userId);
    return user;
  }

  Future uploadFile(file, type) async {
    try {
      List<CloudinaryResponse> responseList;
      CloudinaryResponse response;
      if (type == "Photo") {
        responseList = await cloudinary.multiUpload(
          images
              .map(
                (image) => CloudinaryFile.fromFutureByteData(
                  image.getByteData(),
                  identifier: image.identifier,
                ),
              )
              .toList(),
        );
        responseList.forEach((e) {
          this.jsonUrl += e.url + ",";
        });
        this.jsonUrl = this.jsonUrl.substring(0, this.jsonUrl.length - 1);
      } else if (type == "Video") {
        response = await cloudinary.uploadFile(
          CloudinaryFile.fromFile(file.path,
              resourceType: CloudinaryResourceType.Video),
        );
      }
      setState(() {
        fileUrl = response != null ? response.url : this.jsonUrl;
      });
    } catch (e) {
      print("ERR---------------" + e.toString());
    }
  }

  @override
  Widget build(BuildContext context) {
    return ScreenUtilInit(
      designSize: Size(393.0, 534.0),
      allowFontScaling: false,
      builder: () => LayoutBuilder(
        builder: (_, BoxConstraints constraints) {
          return MaterialApp(
            home: Scaffold(
                appBar: PreferredSize(
                    preferredSize: Size.fromHeight(70.0),
                    child: AppBar(
                      backgroundColor: Color(0xFF083356),
                      leading: GestureDetector(
                        onTap: () {
                          Navigator.pop(context);
                        },
                        child: Padding(
                          padding:
                              EdgeInsets.only(top: ScreenUtil().setWidth(9)),
                          child:
                              Icon(Icons.close, size: 27, color: Colors.white),
                        ),
                      ),
                      title: Container(
                        alignment: Alignment(-0.3, 0),
                        child: Padding(
                          padding:
                              EdgeInsets.only(top: ScreenUtil().setWidth(9)),
                          child: Text(
                            "Share post",
                            textAlign: TextAlign.center,
                            style: TextStyle(
                              color: Colors.white,
                            ),
                          ),
                        ),
                      ),
                    )),
                body: ListView.separated(
                  itemCount: 1,
                  itemBuilder: (BuildContext context, int index) {
                    return _buildContent(this.widget.currentUser);
                  },
                  separatorBuilder: (BuildContext context, int index) =>
                      const Divider(),
                )),
          );
        },
      ),
    );
  }

  _buildContent(currentUser) {
    return Form(
      key: _formKey,
      // child: Container(
      // child: Column(
      // children: [
      // Container(
      // child:
      child: Column(
        children: [
          _buildProfile(currentUser),
          _buildTitle(),
          _buildDescription(),
          images.length > 0
              ? Container(
                  child: _buildGridView(),
                  height: ScreenUtil().setHeight(240),
                )
              : _gif != null
                  ? Container(
                      child: _buildGifContainer(),
                      height: ScreenUtil().setHeight(240),
                    )
                  : Container(
                      child: _buildVideoContainer(),
                      height: ScreenUtil().setHeight(240),
                    ),
          Container(
            alignment: Alignment.bottomCenter,
            padding: EdgeInsets.all(ScreenUtil().setWidth(16)),
            decoration: BoxDecoration(
              color: Colors.white,
              boxShadow: <BoxShadow>[
                BoxShadow(
                  color: Colors.grey[300],
                  offset: new Offset(5.0, 5.0),
                  blurRadius: 20.0,
                )
              ],
            ),
            child: Row(
              children: [
                Expanded(flex: 2, child: _buildHashtag()),
                Expanded(flex: 5, child: _buildAttachment()),
                Flexible(flex: 1, child: _buildShareButton(currentUser))
              ],
            ),
          ),
        ],
      ),
      // ),
      // ],
      // ),
      // ),
    );
  }

  _buildProfile(currentUser) {
    return Container(
        child: Padding(
            padding: EdgeInsets.only(
              left: ScreenUtil().setHeight(17),
              top: ScreenUtil().setHeight(20),
              bottom: ScreenUtil().setHeight(30),
            ),
            child: Row(children: [
              Column(
                children: [
                  Container(
                    margin: EdgeInsets.only(
                      right: ScreenUtil().setWidth(10),
                    ),
                    child: ClipOval(
                      child: Image.network(
                        currentUser.profileImg,
                        width: ScreenUtil().setWidth(33),
                        height: ScreenUtil().setHeight(25),
                        fit: BoxFit.cover,
                      ),
                    ),
                  ),
                ],
              ),
              Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Container(
                    width:
                        ScreenUtil().screenWidth - ScreenUtil().setWidth(163),
                    child: Text(
                      currentUser.firstName + " " + currentUser.lastName,
                      textAlign: TextAlign.left,
                      maxLines: 1,
                      style: TextStyle(
                        fontWeight: FontWeight.w500,
                        color: Colors.black,
                        fontSize: ScreenUtil().setSp(15),
                        fontFamily: 'Rubik',
                      ),
                    ),
                  ),
                  Container(
                    width:
                        ScreenUtil().screenWidth - ScreenUtil().setWidth(165),
                    child: Text(
                      currentUser.about,
                      textAlign: TextAlign.left,
                      maxLines: 1,
                      style: TextStyle(
                        fontWeight: FontWeight.normal,
                        color: Colors.black.withOpacity(0.4),
                        fontSize: ScreenUtil().setSp(13),
                        fontFamily: "Rubik",
                      ),
                    ),
                  ),
                ],
              ),
            ])));
  }

  _buildTitle() {
    return Container(
      padding: EdgeInsets.only(
        left: ScreenUtil().setHeight(17),
        bottom: ScreenUtil().setHeight(17),
      ),
      child: TextFormField(
        style: TextStyle(fontSize: 15),
        minLines: 1,
        maxLines: 1,
        decoration: InputDecoration.collapsed(hintText: "Please type title"),
        controller: titleController,
      ),
    );
  }

  _buildDescription() {
    return Container(
      padding: EdgeInsets.only(
        left: ScreenUtil().setHeight(17),
      ),
      child: HashTagTextField(
        keyboardType: TextInputType.multiline,
        maxLines: null,
        decoration: InputDecoration.collapsed(
            hintText: "Please type all you want to talk about..."),
        controller: descriptionController,
        minLines: 6,
        decoratedStyle: TextStyle(
            fontSize: 15, color: Colors.black, fontWeight: FontWeight.bold),
        basicStyle: TextStyle(fontSize: 15, color: Colors.black),
      ),

      // TextFormField(
      //     style: TextStyle(fontSize: 15),
      //     minLines: 6,
      //     keyboardType: TextInputType.multiline,
      //     maxLines: null,
      //     decoration: InputDecoration.collapsed(
      //         hintText: "Please type all you want to talk about..."),
      //     controller: descriptionController)
    );
  }

  _buildVideoContainer() {
    return Container(
      height: ScreenUtil().setHeight(100),
      child: fileMedia == null
          ? Container()
          : (source == MediaSource.image
              ? Container()
              : VideoWidget(fileMedia)),
    );
  }

  _buildGifContainer() {
    return Container(
      child: Center(child: GiphyImage.original(gif: _gif)),
    );
  }

  _buildAttachment() {
    return Container(
      child: Container(
        child: Column(
          children: [
            Container(
              child: Row(
                children: [
                  this.fileMedia != null
                      ? Row(children: [
                          IconButton(
                            icon: const Icon(
                              Icons.videocam_outlined,
                              color: Colors.black,
                            ),
                            onPressed: () => capture(MediaSource.video),
                          ),
                          Container(
                            child: IconButton(
                                icon: Icon(Icons.camera_alt_outlined),
                                onPressed: null),
                          ),
                        ])
                      : images.length == 0 && this.fileMedia == null
                          ? Row(
                              children: [
                                IconButton(
                                  icon: const Icon(
                                    Icons.videocam_outlined,
                                    color: Colors.black,
                                  ),
                                  onPressed: () => capture(MediaSource.video),
                                ),
                                Container(
                                  child: IconButton(
                                      icon: Icon(Icons.camera_alt_outlined),
                                      onPressed: loadAssets),
                                ),
                              ],
                            )
                          : Row(
                              children: [
                                IconButton(
                                  icon: const Icon(
                                    Icons.videocam_outlined,
                                    color: Colors.grey,
                                  ),
                                  onPressed: null,
                                ),
                                Container(
                                  child: IconButton(
                                      icon: Icon(Icons.camera_alt_outlined),
                                      onPressed: loadAssets),
                                ),
                              ],
                            ),
                  Container(
                    child: IconButton(
                      icon: Icon(Icons.gif),
                      onPressed: () async {
                        GiphyGif res = await GiphyPicker.pickGif(
                          context: context,
                          fullScreenDialog: false,
                          showPreviewPage: true,
                          apiKey: 'EwNGOsJ61NlXaGQ3Nw6ZwxYjZNTyRoW2',
                          decorator: GiphyDecorator(
                            showAppBar: false,
                            searchElevation: 4,
                            giphyTheme: ThemeData.dark(),
                          ),
                        );
                        setState(() {
                          _gif = res;
                        });
                      },
                    ),
                  ),
                  Container(
                    child: IconButton(
                        icon: Icon(Icons.important_devices_outlined),
                        onPressed: getimageditor),
                  ),
                ],
              ),
            )
          ],
        ),
      ),
    );
  }

  Container _buildHashtag() {
    return Container(
      alignment: Alignment.bottomLeft,
      child: TextButton(
        onPressed: () {
          descriptionController.text += ("#");
          descriptionController.selection = TextSelection.fromPosition(
              TextPosition(offset: descriptionController.text.length));
        },
        child: Text("#Add hashtag"),
      ),
    );
  }

  _buildShareButton(currentUser) {
    // var flatButton =
    return Container(
        width: ScreenUtil().setHeight(50),
        child: ClipOval(
            child: Material(
                color: Color(0xFF3DD598),
                child: InkWell(
                  splashColor: Color(0xFFFCA64F),
                  child: IconButton(
                    icon: Icon(FontAwesomeIcons.telegramPlane),
                    color: Colors.white,
                    iconSize: 35,
                    onPressed: () async {
                      if (_formKey.currentState.validate()) {
                        if (images.length > 0) {
                          pluginTitle = "Photo";
                          await uploadFile(images, pluginTitle);
                        } else if (this.fileMedia != null) {
                          pluginTitle = "Video";
                          await uploadFile(this.fileMedia, pluginTitle);
                        } else if (_gif != null) {
                          pluginTitle = "Photo";
                          fileUrl = _gif.images.original.url;
                        }
                        feedCubit.feedRepository.addPost(
                            currentUser,
                            titleController.text,
                            descriptionController.text,
                            pluginTitle,
                            fileUrl);
                        Navigator.pop(context);
                      }
                    },
                  ),
                ))));
  }
}
