import 'dart:core';
import 'dart:convert';
import 'dart:developer';
import 'package:plugin_codelab_new/models/user_model.dart';
import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:http/http.dart';
import 'package:plugin_codelab_new/exeptions/auth_exception.dart';
import '../environment_config.dart';

class HttpService {
  final String apiURL = 'https://${EnvironmentConfig.API_URL}';
  String _jwt;

  HttpService(this._jwt);

  Future<Response> get(url, {Map<String, String> headers}) async {
    final response = await http.get(url, headers: headers);
    if (response.statusCode == 401) {
      if (json.decode(response.body)['message'] == 'Expired JWT Token') {
        throw ExpiredTokenException(response.reasonPhrase);
      } else {
        throw AuthenticateException(response.reasonPhrase);
      }
    } else {
      return response;
    }
  }

  Future<Response> post(url,
      {Map<String, String> headers, body, Encoding encoding}) async {
    final response =
        await http.post(url, headers: headers, body: body, encoding: encoding);
    if (response.statusCode == 401 &&
        json.decode(response.body)['message'] == 'Expired JWT Token') {
      throw ExpiredTokenException(response.reasonPhrase);
    } else {
      return response;
    }
  }

  Future<Response> put(url,
      {Map<String, String> headers, body, Encoding encoding}) async {
    final response =
        await http.put(url, headers: headers, body: body, encoding: encoding);
    if (response.statusCode == 401 &&
        json.decode(response.body)['message'] == 'Expired JWT Token') {
      throw ExpiredTokenException(response.reasonPhrase);
    } else {
      return response;
    }
  }

  Future<http.Response> fetchUser(id) {
    final userId = id;
    return get(Uri.parse('$apiURL/users/getById?id=$userId'), headers: {
      "token":
          "cnejwi8c9wehnd8fchni2x239jxexnjx2n23i8xn3oifr0cwd32fjnrf453vreve"
    });
  }

  Future<http.Response> fetchAllPosts() {
    return get(Uri.parse('$apiURL/posts/all'), headers: {
      "token":
          "cnejwi8c9wehnd8fchni2x239jxexnjx2n23i8xn3oifr0cwd32fjnrf453vreve"
    });
  }

  Future<http.Response> fetchPostsByUserId(id) async {
    return get(Uri.parse('$apiURL/posts/byUserId?user=$id'), headers: {
      "token":
          "cnejwi8c9wehnd8fchni2x239jxexnjx2n23i8xn3oifr0cwd32fjnrf453vreve"
    });
  }

  Future<http.Response> fetchFavoritePosts(currentUserId, saveId) async {
    return get(
        Uri.parse(
            '$apiURL/reports/favorite?user=$currentUserId&saveId=$saveId'),
        headers: {
          "token":
              "cnejwi8c9wehnd8fchni2x239jxexnjx2n23i8xn3oifr0cwd32fjnrf453vreve"
        });
  }

  Future<http.Response> fetchCommentsByPostId(postId) async {
    return get(Uri.parse('$apiURL/post-comments/byPostId?post=$postId'),
        headers: {
          "token":
              "cnejwi8c9wehnd8fchni2x239jxexnjx2n23i8xn3oifr0cwd32fjnrf453vreve"
        });
  }

  Future<http.Response> addPost(
      {@required UserModel user,
      @required String title,
      @required String description,
      @required String type,
      String attachedUrl}) {
    var data = json.encode({
      "user": user.id,
      "description": description,
      "type": type,
      "attachedUrl": attachedUrl,
      "title": title
    });
    log(data);

    return post(Uri.parse('$apiURL/posts/create'),
        headers: {
          "token":
              "cnejwi8c9wehnd8fchni2x239jxexnjx2n23i8xn3oifr0cwd32fjnrf453vreve",
          "content-type": 'application/json',
        },
        body: data);
  }

  Future<http.Response> uploadComment({
    @required int userId,
    @required int postId,
    @required String comment,
  }) {
    var data = json.encode({
      "user": userId,
      "post": postId,
      "comment": comment,
    });
    log(data);

    return post(Uri.parse('$apiURL/post-comments/create'),
        headers: {
          "token":
              "cnejwi8c9wehnd8fchni2x239jxexnjx2n23i8xn3oifr0cwd32fjnrf453vreve",
          "content-type": 'application/json',
        },
        body: data);
  }

  Future<http.Response> fetchAllReports() {
    return get(Uri.parse('$apiURL/report-types/all'), headers: {
      "token":
          "cnejwi8c9wehnd8fchni2x239jxexnjx2n23i8xn3oifr0cwd32fjnrf453vreve"
    });
  }

  Future<http.Response> fetchSubReport(reportTypeId) async {
    return get(
        Uri.parse(
            '$apiURL/report-sub-types/byReportTypeId?reportType=$reportTypeId'),
        headers: {
          "token":
              "cnejwi8c9wehnd8fchni2x239jxexnjx2n23i8xn3oifr0cwd32fjnrf453vreve"
        });
  }

  Future<http.Response> fetchReportByPostId(id) async {
    return await get(Uri.parse('$apiURL/reports/byPostId?post=$id'), headers: {
      "token":
          "cnejwi8c9wehnd8fchni2x239jxexnjx2n23i8xn3oifr0cwd32fjnrf453vreve"
    });
  }

  Future<http.Response> addReport(
      {@required int typeId, @required int userId, @required int postId}) {
    var data = json.encode({"type": typeId, "user": userId, "post": postId});
    log(data);
    return post(Uri.parse('$apiURL/reports/create'),
        headers: {
          "token":
              "cnejwi8c9wehnd8fchni2x239jxexnjx2n23i8xn3oifr0cwd32fjnrf453vreve",
          "content-type": 'application/json',
        },
        body: data);
  }

  Future<http.Response> updatePostReport({@required int postId}) {
    return put(Uri.parse('$apiURL/posts/reports?post=$postId'), headers: {
      "token":
          "cnejwi8c9wehnd8fchni2x239jxexnjx2n23i8xn3oifr0cwd32fjnrf453vreve",
      "content-type": 'application/json',
    });
  }

  Future<http.Response> removeReport({@required reportId}) {
    return delete(Uri.parse('$apiURL/reports/remove?id=$reportId'), headers: {
      "token":
          "cnejwi8c9wehnd8fchni2x239jxexnjx2n23i8xn3oifr0cwd32fjnrf453vreve"
    });
  }

  Future<http.Response> removeComment({@required commentId}) {
    return delete(Uri.parse('$apiURL/post-comments/remove?id=$commentId'),
        headers: {
          "token":
              "cnejwi8c9wehnd8fchni2x239jxexnjx2n23i8xn3oifr0cwd32fjnrf453vreve"
        });
  }

  Future<http.Response> fetchAllSubType() {
    return get(Uri.parse('$apiURL/report-sub-types/all'), headers: {
      "token":
          "cnejwi8c9wehnd8fchni2x239jxexnjx2n23i8xn3oifr0cwd32fjnrf453vreve"
    });
  }

  Future<http.Response> authenticateUser(
      {@required firstName,
      @required lastName,
      @required about,
      @required profileImg,
      @required token}) {
    return get(
        Uri.parse(
            '$apiURL/users/authenticate?firstName=$firstName&lastName=$lastName&about=$about&profileImg=$profileImg&token=$token'),
        headers: {
          "token":
              "cnejwi8c9wehnd8fchni2x239jxexnjx2n23i8xn3oifr0cwd32fjnrf453vreve"
        });
  }
}
