import 'dart:convert';
import 'dart:ui';

import 'package:plugin_codelab_new/pages/comments/screens/chat.dart';
import 'package:plugin_codelab_new/pages/likes_and_dislikes.dart';
import 'package:plugin_codelab_new/widgets/parsed_text.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:plugin_codelab_new/bloc/feed_cubit.dart';
import 'package:plugin_codelab_new/models/post_model.dart';
import 'package:plugin_codelab_new/models/report_model.dart';
import 'package:plugin_codelab_new/models/user_model.dart';
import 'package:plugin_codelab_new/pages/user_posts.dart';
import 'package:plugin_codelab_new/widgets/basics/network_player_widget.dart';
import 'package:plugin_codelab_new/widgets/popup.dart';
import 'package:story/story_page_view/story_page_view.dart';
import 'package:timeago/timeago.dart' as timeago;
import 'package:flutter/material.dart';
import 'package:pk_skeleton/pk_skeleton.dart';
import 'package:hash_at_links_detector/hash_at_links_detector.dart';
import 'package:flutter_share/flutter_share.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_screenutil/screenutil_init.dart';
import 'package:number_display/number_display.dart';
import 'package:status_alert/status_alert.dart';

final display = createDisplay(
  length: 4,
  separator: ',',
  decimal: 2,
);

class FeedPost extends StatefulWidget {
  const FeedPost({
    Key key,
    @required this.postData,
    @required this.currentUser,
    @required this.likeId,
    @required this.dislikeId,
    @required this.saveId,
    @required this.images,
    @required this.commentsLength,
  }) : super(key: key);

  final PostModel postData;
  final UserModel currentUser;
  final int likeId;
  final int dislikeId;
  final int saveId;
  final List<String> images;
  final int commentsLength;

  @override
  _FeedPostState createState() => _FeedPostState();
}

class _FeedPostState extends State<FeedPost> {
  FeedCubit feedCubit;

  List<ReportModel> likesList;
  List<ReportModel> dislikesList;

  bool mySave;
  bool myLike;
  bool myDislike;
  bool flagReport = true;

  int commentsOnPost;
  // int saveReportId = 0;
  // int likeReportId = 0;
  // int disLikeReportId = 0;
  int reportLikeId = 0;
  int reportDislikeId = 0;
  int reportSaveId = 0;

  UserModel userData;
  // ReportModel newReport;

  @override
  void initState() {
    feedCubit = BlocProvider.of<FeedCubit>(context);

    likesList = [];
    dislikesList = [];

    mySave = false;
    myLike = false;
    myDislike = false;

    commentsOnPost = 0;
    userData = this.widget.postData.user;
    getLikesListByPostId(this.widget.postData.id).then(
      (data) {
        setState(
          () {},
        );
      },
    );
    super.initState();
  }

  Future<void> getLikesListByPostId(postId) async {
    List<ReportModel> reportsLikeList = [];
    List<ReportModel> reportsDislikeList = [];
    List<ReportModel> reportsSaveList = [];

    bool isDislike = false;
    bool isLike = false;
    bool isSave = false;

    await feedCubit.feedRepository
        .fetchPostReports(postId)
        .then((reportsList) async => {
              reportsList.forEach((e) {
                if (e.type.id == this.widget.likeId) {
                  reportsLikeList.add(e);
                  if (e.user.id == this.widget.currentUser.id) {
                    isLike = true;
                    reportLikeId = e.id;
                  }
                } else if (e.type.id == this.widget.dislikeId) {
                  reportsDislikeList.add(e);
                  if (e.user.id == this.widget.currentUser.id) {
                    isDislike = true;
                    reportDislikeId = e.id;
                  }
                } else if (e.type.id == this.widget.saveId) {
                  reportsSaveList.add(e);
                  if (e.user.id == this.widget.currentUser.id) {
                    isSave = true;
                    reportSaveId = e.id;
                  }
                } else {
                  if (e.user.id == this.widget.currentUser.id) {
                    this.setState(() {
                      flagReport = false;
                    });
                  }
                }
              }),
              likesList = reportsLikeList,
              dislikesList = reportsDislikeList,
              myLike = isLike,
              myDislike = isDislike,
              mySave = isSave
            });

    await feedCubit.feedRepository
        .fetchPostComments(this.widget.postData.id)
        .then((comments) => {commentsOnPost = comments.length});
  }

  @override
  void dispose() {
    super.dispose();
  }

  Future<void> removeReportPost(reportId, subTypeId) async {
    List<ReportModel> dislikes = [];
    List<ReportModel> likes = [];
    await feedCubit.feedRepository.removeReport(reportId).then((value) => {
          if (subTypeId == this.widget.likeId)
            {
              likesList.forEach((like) {
                if (like.id != reportId) {
                  likes.add(like);
                }
              }),
              setState(() {
                likesList = likes;
              }),
            }
          else if (subTypeId == this.widget.dislikeId)
            {
              dislikesList.forEach((dislike) {
                if (dislike.id != reportId) {
                  dislikes.add(dislike);
                }
              }),
              setState(() {
                dislikesList = dislikes;
              }),
            }
        });

    await feedCubit.feedRepository.removeReport(reportId);
  }

  Future<void> reportPost(subTypeId) async {
    await feedCubit.feedRepository
        .addReport(
            subTypeId, this.widget.currentUser.id, this.widget.postData.id)
        .then((data) => {
              if (subTypeId == this.widget.likeId)
                {
                  setState(() {
                    reportLikeId = data.id;
                    likesList.add(data);
                  }),
                }
              else
                {
                  if (subTypeId == this.widget.dislikeId)
                    {
                      setState(() {
                        reportDislikeId = data.id;
                        dislikesList.add(data);
                      }),
                    }
                  else
                    {
                      if (subTypeId == this.widget.saveId)
                        {
                          reportSaveId = data.id,
                        }
                    }
                }
            });
  }

  Future<void> show() async {
    StatusAlert.show(
      context,
      duration: Duration(seconds: 2),
      title: 'Save post',
      subtitle: 'this post saved for later',
      configuration: IconConfiguration(
        icon: Icons.favorite_border_rounded,
        size: 80,
      ),
    );
    await reportPost(this.widget.saveId);
  }

  @override
  Widget build(BuildContext context) {
    Widget buildNetwork(url) => NetworkPlayerWidget(url: url);

    this.widget.images.forEach((image) {});
    Future<void> share(title, description, attachedUrl, user) async {
      await FlutterShare.share(
          title: 'Post sharing',
          text: title +
              ' \n \n' +
              description +
              ' \n (' +
              user.firstName +
              " " +
              user.lastName +
              ")",
          linkUrl: attachedUrl,
          chooserTitle: 'Post sharing');
    }

    final date2 = DateTime.now();
    final difference =
        date2.difference(this.widget.postData.createdAt).inMinutes;
    final fifteenAgo =
        new DateTime.now().subtract(new Duration(minutes: difference));
    return likesList == []
        ? PKCardSkeleton(
            isCircularImage: true,
            isBottomLinesActive: true,
          )
        : Container(
            color: Colors.white,
            child: userData == null
                ? PKCardSkeleton(
                    isCircularImage: true,
                    isBottomLinesActive: true,
                  )
                : Container(
                    padding: EdgeInsets.symmetric(
                        horizontal: ScreenUtil().setWidth(1.0)),
                    child: Container(
                      margin: EdgeInsets.only(
                          top: ScreenUtil().setWidth(23.0),
                          bottom: ScreenUtil().setWidth(8)),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.center,
                        mainAxisAlignment: MainAxisAlignment.center,
                        children: <Widget>[
                          Row(
                            children: [
                              Column(
                                children: [
                                  Container(
                                      margin: EdgeInsets.only(
                                          left: ScreenUtil().setWidth(12.0)),
                                      child: GestureDetector(
                                        onTap: () {
                                          Navigator.push(
                                            context,
                                            MaterialPageRoute(
                                              builder: (context) => UserPost(
                                                userData: userData,
                                                currentUser:
                                                    this.widget.currentUser,
                                              ),
                                            ),
                                          );
                                        },
                                        child: ClipOval(
                                          child: Image.network(
                                            userData.profileImg,
                                            width: ScreenUtil().setWidth(40),
                                            height: ScreenUtil().setHeight(30),
                                            fit: BoxFit.cover,
                                          ),
                                        ),
                                      )),
                                ],
                              ),
                              Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                children: [
                                  Container(
                                    width: ScreenUtil().setWidth(250),
                                    margin: EdgeInsets.only(
                                        left: ScreenUtil().setWidth(10.0)),
                                    child: Text(
                                      userData.firstName +
                                          " " +
                                          userData.lastName,
                                      textAlign: TextAlign.left,
                                      maxLines: 1,
                                      style: TextStyle(
                                        fontWeight: FontWeight.w500,
                                        color: Colors.black,
                                        fontSize: ScreenUtil().setSp(15),
                                        fontFamily: 'Rubik',
                                      ),
                                    ),
                                  ),
                                  Container(
                                    width: ScreenUtil().setWidth(250),
                                    margin: EdgeInsets.only(
                                        left: ScreenUtil().setWidth(10.0)),
                                    child: Text(
                                      userData.about,
                                      textAlign: TextAlign.left,
                                      maxLines: 1,
                                      style: TextStyle(
                                        fontWeight: FontWeight.normal,
                                        color: Color(0xff98A9BC),
                                        fontSize: ScreenUtil().setSp(13),
                                        fontFamily: "Rubik",
                                      ),
                                    ),
                                  ),
                                  Container(
                                    margin: EdgeInsets.only(
                                        left: ScreenUtil().setWidth(10.0)),
                                    child: Text(
                                      timeago.format(fifteenAgo,
                                          locale: 'en_short'),
                                      style:
                                          TextStyle(color: Color(0xff98A9BC)),
                                    ),
                                  )
                                ],
                              ),
                              Column(
                                children: [
                                  Container(
                                      child: IconButton(
                                    icon: Icon(Icons.more_vert),
                                    iconSize: 35,
                                    onPressed: () {
                                      showModalBottomSheet(
                                        context: context,
                                        builder: (context) {
                                          return Wrap(
                                            children: [
                                              ListTile(
                                                leading: Icon(
                                                  Icons.bookmarks_outlined,
                                                  color: Colors.black,
                                                ),
                                                title: Text(
                                                    'Save this item for later'),
                                              ),
                                              ListTile(
                                                leading: Icon(
                                                  Icons.share_outlined,
                                                  color: Colors.black,
                                                ),
                                                title: Text('Share via'),
                                              ),
                                              ListTile(
                                                leading: Icon(
                                                  Icons.cancel,
                                                  color: Colors.black,
                                                ),
                                                title:
                                                    Text('Unfollow Janet Doe'),
                                              ),
                                              ListTile(
                                                onTap: flagReport == true
                                                    ? () {
                                                        showDialog(
                                                          context: context,
                                                          builder: (BuildContext
                                                              context) {
                                                            return Reports(
                                                              currentUser: this
                                                                  .widget
                                                                  .currentUser,
                                                              postData: this
                                                                  .widget
                                                                  .postData,
                                                              onClick: () {
                                                                Navigator.of(
                                                                        context)
                                                                    .pop();
                                                              },
                                                            );
                                                          },
                                                        );
                                                      }
                                                    : null,
                                                leading: flagReport == true
                                                    ? Icon(
                                                        Icons
                                                            .outlined_flag_rounded,
                                                        color: Colors.black,
                                                      )
                                                    : Icon(
                                                        Icons
                                                            .outlined_flag_rounded,
                                                        color: Colors.grey,
                                                      ),
                                                title: flagReport == true
                                                    ? Text('Report the post')
                                                    : Text(
                                                        'Report the post',
                                                        style: TextStyle(
                                                            color: Colors.grey),
                                                      ),
                                              ),
                                            ],
                                          );
                                        },
                                      );
                                    },
                                    // child: null,
                                  ))
                                ],
                              )
                            ],
                          ),
                          Container(
                            margin: EdgeInsets.only(
                                top: ScreenUtil().setWidth(10),
                                left: ScreenUtil().setWidth(12.0)),
                            width: ScreenUtil().screenWidth,
                            child: Parse(
                              myText: this.widget.postData.title,
                              style: TextStyle(
                                  fontSize: ScreenUtil().setSp(15),
                                  fontFamily: 'Rubik',
                                  color: Colors.black),
                            ),
                          ),
                          Container(
                            margin: EdgeInsets.only(
                                left: ScreenUtil().setWidth(12.0)),
                            width: ScreenUtil().screenWidth,
                            child: Parse(
                              myText: this.widget.postData.description,
                              style: TextStyle(
                                  fontSize: ScreenUtil().setSp(15),
                                  fontFamily: 'Rubik',
                                  color: Colors.black),
                            ),
                          ),
                          Container(
                              margin: EdgeInsets.only(
                                  top: ScreenUtil().setWidth(5)),
                              child: this.widget.postData.type == "Photo"
                                  ? Container(
                                      height: ScreenUtil().setHeight(180),
                                      width: ScreenUtil().screenWidth,
                                      child: this.widget.images.length > 1
                                          ? StoryPageView(
                                              indicatorPadding:
                                                  EdgeInsets.symmetric(
                                                      vertical: 15,
                                                      horizontal: 8),
                                              itemBuilder: (context, pageIndex,
                                                  storyIndex) {
                                                return Image.network(
                                                    this
                                                        .widget
                                                        .images[storyIndex],
                                                    fit: BoxFit.fill);
                                              },
                                              storyLength: (pageIndex) {
                                                return this
                                                    .widget
                                                    .images
                                                    .length;
                                              },
                                              pageLength: 1,
                                            )
                                          : Image.network(
                                              this.widget.postData.attachedUrl,
                                              fit: BoxFit.fill,
                                            ))
                                  // :
                                  // this.widget.postData.type == "Video"
                                  //     ? Container(
                                  //         child: buildNetwork(
                                  //             this.widget.postData.attachedUrl))
                                  : Container()),
                          Column(
                            children: [
                              Divider(
                                color: Color(0xff98A9BC),
                              ),
                            ],
                          ),
                          Container(
                            margin: EdgeInsets.only(
                                left: ScreenUtil().setWidth(12.0)),
                            child: Row(
                              children: [
                                Container(
                                  width: ScreenUtil().setWidth(175),
                                  margin: EdgeInsets.only(
                                    top: ScreenUtil().setWidth(8),
                                    bottom: ScreenUtil().setWidth(8),
                                  ),
                                  child: GestureDetector(
                                    onTap: () {
                                      Navigator.push(
                                          context,
                                          MaterialPageRoute(
                                            builder: (context) => ChatScreen(
                                              currentUser:
                                                  this.widget.currentUser,
                                              senderName: this
                                                      .widget
                                                      .currentUser
                                                      .firstName +
                                                  " " +
                                                  this
                                                      .widget
                                                      .currentUser
                                                      .lastName,
                                              about:
                                                  this.widget.currentUser.about,
                                              senderImage: this
                                                  .widget
                                                  .currentUser
                                                  .profileImg,
                                              post: FeedPost(
                                                  postData:
                                                      this.widget.postData,
                                                  currentUser:
                                                      this.widget.currentUser,
                                                  likeId: this.widget.likeId,
                                                  saveId: this.widget.saveId,
                                                  dislikeId:
                                                      this.widget.dislikeId,
                                                  images: this.widget.images,
                                                  commentsLength: this
                                                      .widget
                                                      .commentsLength),
                                            ),
                                          ));
                                    },
                                    child: this.widget.commentsLength > 0
                                        ? Text(
                                            "Show all " +
                                                display(this
                                                    .widget
                                                    .commentsLength) +
                                                " comments",
                                            style: TextStyle(
                                                fontSize:
                                                    ScreenUtil().setSp(13),
                                                color: Color(0xff98A9BC)),
                                          )
                                        : this.commentsOnPost > 0
                                            ? Text(
                                                "Show all " +
                                                    display(commentsOnPost) +
                                                    " comments",
                                                style: TextStyle(
                                                    fontSize:
                                                        ScreenUtil().setSp(13),
                                                    color: Color(0xff98A9BC)),
                                              )
                                            : Text(""),
                                  ),
                                ),
                                Container(
                                  alignment: Alignment.centerRight,
                                  width: ScreenUtil().setWidth(180),
                                  child: GestureDetector(
                                    onTap: () {
                                      Navigator.push(
                                        context,
                                        MaterialPageRoute(
                                          builder: (context) =>
                                              LikesAndDislikes(
                                                  likesList: likesList,
                                                  dislikesList: dislikesList,
                                                  postTitle: this
                                                      .widget
                                                      .postData
                                                      .title),
                                        ),
                                      );
                                    },
                                    child:
                                        likesList.length + dislikesList.length >
                                                0
                                            ? Text(
                                                "Show " +
                                                    display(likesList.length +
                                                        dislikesList.length) +
                                                    " Likes&Dislikes",
                                                style: TextStyle(
                                                    fontSize:
                                                        ScreenUtil().setSp(13),
                                                    color: Color(0xff98A9BC)),
                                              )
                                            : Text(""),
                                  ),
                                ),
                              ],
                            ),
                          ),
                          Column(
                            children: [
                              Divider(
                                color: Color(0xff98A9BC),
                              ),
                            ],
                          ),
                          Container(
                            child: Row(
                              children: [
                                Expanded(
                                    child: Column(children: [
                                  IconButton(
                                    icon: myLike
                                        ? Icon(Icons.thumb_up,
                                            color: Colors.green)
                                        : Icon(Icons.thumb_up_off_alt,
                                            color: Colors.black),
                                    onPressed: myDislike
                                        ? null
                                        : () {
                                            setState(
                                              () {
                                                myLike = !myLike;
                                                if (myLike) {
                                                  reportPost(
                                                      this.widget.likeId);
                                                  if (myDislike) {
                                                    myDislike = false;
                                                    removeReportPost(
                                                        reportDislikeId,
                                                        this.widget.dislikeId);
                                                  }
                                                } else {
                                                  removeReportPost(reportLikeId,
                                                      this.widget.likeId);
                                                }
                                              },
                                            );
                                          },
                                  ),
                                  Text(
                                    "Like",
                                    style: TextStyle(
                                        fontSize: ScreenUtil().setSp(12)),
                                  )
                                ])),
                                Expanded(
                                  child: Column(children: [
                                    IconButton(
                                      icon: myDislike
                                          ? Icon(Icons.thumb_down,
                                              color: Colors.red)
                                          : Icon(
                                              Icons.thumb_down_off_alt,
                                              color: Colors.black,
                                            ),
                                      onPressed: myLike
                                          ? null
                                          : () {
                                              setState(
                                                () {
                                                  myDislike = !myDislike;
                                                  if (myDislike) {
                                                    reportPost(
                                                        this.widget.dislikeId);
                                                    if (myLike) {
                                                      myLike = false;
                                                      removeReportPost(
                                                          reportLikeId,
                                                          this.widget.likeId);
                                                    }
                                                  } else {
                                                    removeReportPost(
                                                        reportDislikeId,
                                                        this.widget.dislikeId);
                                                  }
                                                },
                                              );
                                            },
                                    ),
                                    Text(
                                      "Dislike",
                                      style: TextStyle(
                                          fontSize: ScreenUtil().setSp(12)),
                                    )
                                  ]),
                                ),
                                Expanded(
                                    child: Column(children: [
                                  IconButton(
                                      icon: Icon(
                                        Icons.messenger_outline,
                                        color: Colors.black,
                                      ),
                                      onPressed: () {
                                        Navigator.push(
                                            context,
                                            MaterialPageRoute(
                                              builder: (context) => ChatScreen(
                                                currentUser:
                                                    this.widget.currentUser,
                                                senderName: this
                                                        .widget
                                                        .currentUser
                                                        .firstName +
                                                    " " +
                                                    this
                                                        .widget
                                                        .currentUser
                                                        .lastName,
                                                about: this
                                                    .widget
                                                    .currentUser
                                                    .about,
                                                senderImage: this
                                                    .widget
                                                    .currentUser
                                                    .profileImg,
                                                post: FeedPost(
                                                    postData:
                                                        this.widget.postData,
                                                    currentUser:
                                                        this.widget.currentUser,
                                                    likeId: this.widget.likeId,
                                                    saveId: this.widget.saveId,
                                                    dislikeId:
                                                        this.widget.dislikeId,
                                                    images: this.widget.images,
                                                    commentsLength: this
                                                        .widget
                                                        .commentsLength),
                                              ),
                                            ));
                                      }),
                                  Text(
                                    "Comments",
                                    style: TextStyle(
                                        fontSize: ScreenUtil().setSp(12)),
                                  )
                                ])),
                                Expanded(
                                    child: Column(children: [
                                  TextButton(
                                    child: Icon(
                                      Icons.share_outlined,
                                      color: Colors.black,
                                    ),
                                    onPressed: () {
                                      share(
                                          this.widget.postData.title,
                                          this.widget.postData.description,
                                          this.widget.postData.attachedUrl,
                                          userData);
                                    },
                                  ),
                                  Text(
                                    "Share",
                                    style: TextStyle(
                                        fontSize: ScreenUtil().setSp(12)),
                                  )
                                ])),
                                Expanded(
                                    child: Column(children: [
                                  IconButton(
                                      icon: Icon(
                                        mySave
                                            ? Icons.favorite
                                            : Icons.favorite_outline,
                                        color:
                                            mySave ? Colors.pink : Colors.black,
                                      ),
                                      onPressed: () {
                                        setState(() {
                                          mySave = !mySave;
                                        });
                                        mySave
                                            ? show()
                                            : removeReportPost(reportSaveId,
                                                this.widget.saveId);
                                      }),
                                  Text(
                                    "Save",
                                    style: TextStyle(
                                        fontSize: ScreenUtil().setSp(12)),
                                  )
                                ])),
                              ],
                            ),
                          ),
                        ],
                      ),
                    ),
                  ),
          );
  }
}
